# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import

from ._backup_usage_summaries_crr_operations import BackupUsageSummariesCRROperations  # type: ignore
from ._aad_properties_operations import AadPropertiesOperations  # type: ignore
from ._cross_region_restore_operations import CrossRegionRestoreOperations  # type: ignore
from ._backup_crr_job_details_operations import BackupCrrJobDetailsOperations  # type: ignore
from ._backup_crr_jobs_operations import BackupCrrJobsOperations  # type: ignore
from ._crr_operation_results_operations import CrrOperationResultsOperations  # type: ignore
from ._crr_operation_status_operations import CrrOperationStatusOperations  # type: ignore
from ._recovery_points_operations import RecoveryPointsOperations  # type: ignore
from ._backup_resource_storage_configs_operations import BackupResourceStorageConfigsOperations  # type: ignore
from ._recovery_points_crr_operations import RecoveryPointsCrrOperations  # type: ignore
from ._backup_protected_items_crr_operations import BackupProtectedItemsCrrOperations  # type: ignore

from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "BackupUsageSummariesCRROperations",
    "AadPropertiesOperations",
    "CrossRegionRestoreOperations",
    "BackupCrrJobDetailsOperations",
    "BackupCrrJobsOperations",
    "CrrOperationResultsOperations",
    "CrrOperationStatusOperations",
    "RecoveryPointsOperations",
    "BackupResourceStorageConfigsOperations",
    "RecoveryPointsCrrOperations",
    "BackupProtectedItemsCrrOperations",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()



# pylint: disable=line-too-long,useless-suppression
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from collections.abc import MutableMapping
from typing import Any, Callable, Optional, TypeVar
import urllib.parse

from azure.core import AsyncPipelineClient
from azure.core.async_paging import AsyncItemPaged, AsyncList
from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    map_error,
)
from azure.core.pipeline import PipelineResponse
from azure.core.rest import AsyncHttpResponse, HttpRequest
from azure.core.tracing.decorator import distributed_trace
from azure.core.tracing.decorator_async import distributed_trace_async
from azure.core.utils import case_insensitive_dict
from azure.mgmt.core.exceptions import ARMErrorFormat

from ... import models as _models
from ..._utils.serialization import Deserializer, Serializer
from ...operations._recovery_points_crr_operations import build_get_request, build_list_request
from .._configuration import RecoveryServicesBackupPassiveClientConfiguration

T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, dict[str, Any]], Any]]
List = list


class RecoveryPointsCrrOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.mgmt.recoveryservicesbackup.passivestamp.aio.RecoveryServicesBackupPassiveClient`'s
        :attr:`recovery_points_crr` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client: AsyncPipelineClient = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config: RecoveryServicesBackupPassiveClientConfiguration = (
            input_args.pop(0) if input_args else kwargs.pop("config")
        )
        self._serialize: Serializer = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize: Deserializer = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def list(
        self,
        vault_name: str,
        resource_group_name: str,
        fabric_name: str,
        container_name: str,
        protected_item_name: str,
        filter: Optional[str] = None,
        **kwargs: Any
    ) -> AsyncItemPaged["_models.RecoveryPointResource"]:
        """Lists the backup copies for the backed up item.

        :param vault_name: The name of the recovery services vault. Required.
        :type vault_name: str
        :param resource_group_name: The name of the resource group where the recovery services vault is
         present. Required.
        :type resource_group_name: str
        :param fabric_name: Fabric name associated with the backed up item. Required.
        :type fabric_name: str
        :param container_name: Container name associated with the backed up item. Required.
        :type container_name: str
        :param protected_item_name: Backed up item whose backup copies are to be fetched. Required.
        :type protected_item_name: str
        :param filter: OData filter options. Default value is None.
        :type filter: str
        :return: An iterator like instance of either RecoveryPointResource or the result of
         cls(response)
        :rtype:
         ~azure.core.async_paging.AsyncItemPaged[~azure.mgmt.recoveryservicesbackup.passivestamp.models.RecoveryPointResource]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[_models.RecoveryPointResourceList] = kwargs.pop("cls", None)

        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(next_link=None):
            if not next_link:

                _request = build_list_request(
                    vault_name=vault_name,
                    resource_group_name=resource_group_name,
                    fabric_name=fabric_name,
                    container_name=container_name,
                    protected_item_name=protected_item_name,
                    subscription_id=self._config.subscription_id,
                    filter=filter,
                    api_version=api_version,
                    headers=_headers,
                    params=_params,
                )
                _request.url = self._client.format_url(_request.url)

            else:
                # make call to next link with the client's api-version
                _parsed_next_link = urllib.parse.urlparse(next_link)
                _next_request_params = case_insensitive_dict(
                    {
                        key: [urllib.parse.quote(v) for v in value]
                        for key, value in urllib.parse.parse_qs(_parsed_next_link.query).items()
                    }
                )
                _next_request_params["api-version"] = self._config.api_version
                _request = HttpRequest(
                    "GET", urllib.parse.urljoin(next_link, _parsed_next_link.path), params=_next_request_params
                )
                _request.url = self._client.format_url(_request.url)
                _request.method = "GET"
            return _request

        async def extract_data(pipeline_response):
            deserialized = self._deserialize("RecoveryPointResourceList", pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return deserialized.next_link or None, AsyncList(list_of_elem)

        async def get_next(next_link=None):
            _request = prepare_request(next_link)

            _stream = False
            pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
                _request, stream=_stream, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                error = self._deserialize.failsafe_deserialize(_models.NewErrorResponse, pipeline_response)
                raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

            return pipeline_response

        return AsyncItemPaged(get_next, extract_data)

    @distributed_trace_async
    async def get(
        self,
        vault_name: str,
        resource_group_name: str,
        fabric_name: str,
        container_name: str,
        protected_item_name: str,
        recovery_point_id: str,
        **kwargs: Any
    ) -> _models.RecoveryPointResource:
        """Provides the information of the backed up data identified using RecoveryPointID.

        :param vault_name: The name of the recovery services vault. Required.
        :type vault_name: str
        :param resource_group_name: The name of the resource group where the recovery services vault is
         present. Required.
        :type resource_group_name: str
        :param fabric_name: Fabric name associated with backed up item. Required.
        :type fabric_name: str
        :param container_name: Container name associated with backed up item. Required.
        :type container_name: str
        :param protected_item_name: Backed up item name whose backup data needs to be fetched.
         Required.
        :type protected_item_name: str
        :param recovery_point_id: RecoveryPointID represents the backed up data to be fetched.
         Required.
        :type recovery_point_id: str
        :return: RecoveryPointResource or the result of cls(response)
        :rtype: ~azure.mgmt.recoveryservicesbackup.passivestamp.models.RecoveryPointResource
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[_models.RecoveryPointResource] = kwargs.pop("cls", None)

        _request = build_get_request(
            vault_name=vault_name,
            resource_group_name=resource_group_name,
            fabric_name=fabric_name,
            container_name=container_name,
            protected_item_name=protected_item_name,
            recovery_point_id=recovery_point_id,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.NewErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("RecoveryPointResource", pipeline_response.http_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import


from ._models_py3 import (  # type: ignore
    AADProperties,
    AADPropertiesResource,
    AzureFileShareRecoveryPoint,
    AzureFileShareRestoreRequest,
    AzureFileshareProtectedItem,
    AzureFileshareProtectedItemExtendedInfo,
    AzureIaaSClassicComputeVMProtectedItem,
    AzureIaaSComputeVMProtectedItem,
    AzureIaaSVMErrorInfo,
    AzureIaaSVMHealthDetails,
    AzureIaaSVMJob,
    AzureIaaSVMJobExtendedInfo,
    AzureIaaSVMJobTaskDetails,
    AzureIaaSVMProtectedItem,
    AzureIaaSVMProtectedItemExtendedInfo,
    AzureSqlProtectedItem,
    AzureSqlProtectedItemExtendedInfo,
    AzureStorageErrorInfo,
    AzureStorageJob,
    AzureStorageJobExtendedInfo,
    AzureStorageJobTaskDetails,
    AzureVmWorkloadProtectedItem,
    AzureVmWorkloadProtectedItemExtendedInfo,
    AzureVmWorkloadSAPAseDatabaseProtectedItem,
    AzureVmWorkloadSAPHanaDatabaseProtectedItem,
    AzureVmWorkloadSQLDatabaseProtectedItem,
    AzureWorkloadErrorInfo,
    AzureWorkloadJob,
    AzureWorkloadJobExtendedInfo,
    AzureWorkloadJobTaskDetails,
    AzureWorkloadPointInTimeRecoveryPoint,
    AzureWorkloadPointInTimeRestoreRequest,
    AzureWorkloadRecoveryPoint,
    AzureWorkloadRestoreRequest,
    AzureWorkloadSAPHanaPointInTimeRecoveryPoint,
    AzureWorkloadSAPHanaPointInTimeRestoreRequest,
    AzureWorkloadSAPHanaRecoveryPoint,
    AzureWorkloadSAPHanaRestoreRequest,
    AzureWorkloadSQLPointInTimeRecoveryPoint,
    AzureWorkloadSQLPointInTimeRestoreRequest,
    AzureWorkloadSQLRecoveryPoint,
    AzureWorkloadSQLRecoveryPointExtendedInfo,
    AzureWorkloadSQLRestoreRequest,
    BEKDetails,
    BMSAADPropertiesQueryObject,
    BMSBackupSummariesQueryObject,
    BMSRPQueryObject,
    BackupManagementUsage,
    BackupManagementUsageList,
    BackupResourceConfig,
    BackupResourceConfigResource,
    ClientScriptForConnect,
    CrossRegionRestoreRequest,
    CrossRegionRestoreRequestResource,
    CrrAccessToken,
    CrrAccessTokenResource,
    CrrJobRequest,
    CrrJobRequestResource,
    DPMProtectedItem,
    DPMProtectedItemExtendedInfo,
    DiskExclusionProperties,
    DiskInformation,
    DpmErrorInfo,
    DpmJob,
    DpmJobExtendedInfo,
    DpmJobTaskDetails,
    EncryptionDetails,
    ErrorAdditionalInfo,
    ErrorDetail,
    ExtendedProperties,
    GenericProtectedItem,
    GenericRecoveryPoint,
    IaasVMRecoveryPoint,
    IaasVMRestoreRequest,
    IdentityBasedRestoreDetails,
    IdentityInfo,
    InstantItemRecoveryTarget,
    Job,
    JobQueryObject,
    JobResource,
    JobResourceList,
    KEKDetails,
    KPIResourceHealthDetails,
    KeyAndSecretDetails,
    MabErrorInfo,
    MabFileFolderProtectedItem,
    MabFileFolderProtectedItemExtendedInfo,
    MabJob,
    MabJobExtendedInfo,
    MabJobTaskDetails,
    NameInfo,
    NewErrorResponse,
    NewErrorResponseError,
    OperationStatus,
    OperationStatusError,
    OperationStatusExtendedInfo,
    OperationStatusJobExtendedInfo,
    OperationStatusJobsExtendedInfo,
    OperationStatusProvisionILRExtendedInfo,
    OperationStatusRecoveryPointExtendedInfo,
    PointInTimeRange,
    ProtectedItem,
    ProtectedItemQueryObject,
    ProtectedItemResource,
    ProtectedItemResourceList,
    RecoveryPoint,
    RecoveryPointDiskConfiguration,
    RecoveryPointMoveReadinessInfo,
    RecoveryPointResource,
    RecoveryPointResourceList,
    RecoveryPointTierInformation,
    Resource,
    ResourceHealthDetails,
    ResourceList,
    RestoreFileSpecs,
    RestoreRequest,
    SQLDataDirectory,
    SQLDataDirectoryMapping,
    TargetAFSRestoreInfo,
    TargetRestoreInfo,
    WorkloadCrrAccessToken,
)

from ._recovery_services_backup_passive_client_enums import (  # type: ignore
    BackupManagementType,
    CopyOptions,
    CreateMode,
    DataSourceType,
    HealthState,
    HealthStatus,
    JobOperationType,
    JobStatus,
    JobSupportedAction,
    LastBackupStatus,
    MabServerType,
    OperationStatusValues,
    OverwriteOptions,
    ProtectedItemHealthStatus,
    ProtectedItemState,
    ProtectionState,
    RecoveryMode,
    RecoveryPointTierStatus,
    RecoveryPointTierType,
    RecoveryType,
    ResourceHealthStatus,
    RestorePointQueryType,
    RestorePointType,
    RestoreRequestType,
    SQLDataDirectoryType,
    StorageType,
    StorageTypeState,
    Type,
    UsagesUnit,
    WorkloadType,
)
from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "AADProperties",
    "AADPropertiesResource",
    "AzureFileShareRecoveryPoint",
    "AzureFileShareRestoreRequest",
    "AzureFileshareProtectedItem",
    "AzureFileshareProtectedItemExtendedInfo",
    "AzureIaaSClassicComputeVMProtectedItem",
    "AzureIaaSComputeVMProtectedItem",
    "AzureIaaSVMErrorInfo",
    "AzureIaaSVMHealthDetails",
    "AzureIaaSVMJob",
    "AzureIaaSVMJobExtendedInfo",
    "AzureIaaSVMJobTaskDetails",
    "AzureIaaSVMProtectedItem",
    "AzureIaaSVMProtectedItemExtendedInfo",
    "AzureSqlProtectedItem",
    "AzureSqlProtectedItemExtendedInfo",
    "AzureStorageErrorInfo",
    "AzureStorageJob",
    "AzureStorageJobExtendedInfo",
    "AzureStorageJobTaskDetails",
    "AzureVmWorkloadProtectedItem",
    "AzureVmWorkloadProtectedItemExtendedInfo",
    "AzureVmWorkloadSAPAseDatabaseProtectedItem",
    "AzureVmWorkloadSAPHanaDatabaseProtectedItem",
    "AzureVmWorkloadSQLDatabaseProtectedItem",
    "AzureWorkloadErrorInfo",
    "AzureWorkloadJob",
    "AzureWorkloadJobExtendedInfo",
    "AzureWorkloadJobTaskDetails",
    "AzureWorkloadPointInTimeRecoveryPoint",
    "AzureWorkloadPointInTimeRestoreRequest",
    "AzureWorkloadRecoveryPoint",
    "AzureWorkloadRestoreRequest",
    "AzureWorkloadSAPHanaPointInTimeRecoveryPoint",
    "AzureWorkloadSAPHanaPointInTimeRestoreRequest",
    "AzureWorkloadSAPHanaRecoveryPoint",
    "AzureWorkloadSAPHanaRestoreRequest",
    "AzureWorkloadSQLPointInTimeRecoveryPoint",
    "AzureWorkloadSQLPointInTimeRestoreRequest",
    "AzureWorkloadSQLRecoveryPoint",
    "AzureWorkloadSQLRecoveryPointExtendedInfo",
    "AzureWorkloadSQLRestoreRequest",
    "BEKDetails",
    "BMSAADPropertiesQueryObject",
    "BMSBackupSummariesQueryObject",
    "BMSRPQueryObject",
    "BackupManagementUsage",
    "BackupManagementUsageList",
    "BackupResourceConfig",
    "BackupResourceConfigResource",
    "ClientScriptForConnect",
    "CrossRegionRestoreRequest",
    "CrossRegionRestoreRequestResource",
    "CrrAccessToken",
    "CrrAccessTokenResource",
    "CrrJobRequest",
    "CrrJobRequestResource",
    "DPMProtectedItem",
    "DPMProtectedItemExtendedInfo",
    "DiskExclusionProperties",
    "DiskInformation",
    "DpmErrorInfo",
    "DpmJob",
    "DpmJobExtendedInfo",
    "DpmJobTaskDetails",
    "EncryptionDetails",
    "ErrorAdditionalInfo",
    "ErrorDetail",
    "ExtendedProperties",
    "GenericProtectedItem",
    "GenericRecoveryPoint",
    "IaasVMRecoveryPoint",
    "IaasVMRestoreRequest",
    "IdentityBasedRestoreDetails",
    "IdentityInfo",
    "InstantItemRecoveryTarget",
    "Job",
    "JobQueryObject",
    "JobResource",
    "JobResourceList",
    "KEKDetails",
    "KPIResourceHealthDetails",
    "KeyAndSecretDetails",
    "MabErrorInfo",
    "MabFileFolderProtectedItem",
    "MabFileFolderProtectedItemExtendedInfo",
    "MabJob",
    "MabJobExtendedInfo",
    "MabJobTaskDetails",
    "NameInfo",
    "NewErrorResponse",
    "NewErrorResponseError",
    "OperationStatus",
    "OperationStatusError",
    "OperationStatusExtendedInfo",
    "OperationStatusJobExtendedInfo",
    "OperationStatusJobsExtendedInfo",
    "OperationStatusProvisionILRExtendedInfo",
    "OperationStatusRecoveryPointExtendedInfo",
    "PointInTimeRange",
    "ProtectedItem",
    "ProtectedItemQueryObject",
    "ProtectedItemResource",
    "ProtectedItemResourceList",
    "RecoveryPoint",
    "RecoveryPointDiskConfiguration",
    "RecoveryPointMoveReadinessInfo",
    "RecoveryPointResource",
    "RecoveryPointResourceList",
    "RecoveryPointTierInformation",
    "Resource",
    "ResourceHealthDetails",
    "ResourceList",
    "RestoreFileSpecs",
    "RestoreRequest",
    "SQLDataDirectory",
    "SQLDataDirectoryMapping",
    "TargetAFSRestoreInfo",
    "TargetRestoreInfo",
    "WorkloadCrrAccessToken",
    "BackupManagementType",
    "CopyOptions",
    "CreateMode",
    "DataSourceType",
    "HealthState",
    "HealthStatus",
    "JobOperationType",
    "JobStatus",
    "JobSupportedAction",
    "LastBackupStatus",
    "MabServerType",
    "OperationStatusValues",
    "OverwriteOptions",
    "ProtectedItemHealthStatus",
    "ProtectedItemState",
    "ProtectionState",
    "RecoveryMode",
    "RecoveryPointTierStatus",
    "RecoveryPointTierType",
    "RecoveryType",
    "ResourceHealthStatus",
    "RestorePointQueryType",
    "RestorePointType",
    "RestoreRequestType",
    "SQLDataDirectoryType",
    "StorageType",
    "StorageTypeState",
    "Type",
    "UsagesUnit",
    "WorkloadType",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from copy import deepcopy
from typing import Any, Awaitable, Optional, TYPE_CHECKING, cast
from typing_extensions import Self

from azure.core.pipeline import policies
from azure.core.rest import AsyncHttpResponse, HttpRequest
from azure.core.settings import settings
from azure.mgmt.core import AsyncARMPipelineClient
from azure.mgmt.core.policies import AsyncARMAutoResourceProviderRegistrationPolicy
from azure.mgmt.core.tools import get_arm_endpoints

from .. import models as _models
from .._utils.serialization import Deserializer, Serializer
from ._configuration import RecoveryServicesBackupPassiveClientConfiguration
from .operations import (
    AadPropertiesOperations,
    BackupCrrJobDetailsOperations,
    BackupCrrJobsOperations,
    BackupProtectedItemsCrrOperations,
    BackupResourceStorageConfigsOperations,
    BackupUsageSummariesCRROperations,
    CrossRegionRestoreOperations,
    CrrOperationResultsOperations,
    CrrOperationStatusOperations,
    RecoveryPointsCrrOperations,
    RecoveryPointsOperations,
)

if TYPE_CHECKING:
    from azure.core import AzureClouds
    from azure.core.credentials_async import AsyncTokenCredential


class RecoveryServicesBackupPassiveClient:  # pylint: disable=too-many-instance-attributes
    """Open API 2.0 Specs for Azure RecoveryServices Backup service.

    :ivar backup_usage_summaries_crr: BackupUsageSummariesCRROperations operations
    :vartype backup_usage_summaries_crr:
     azure.mgmt.recoveryservicesbackup.passivestamp.aio.operations.BackupUsageSummariesCRROperations
    :ivar aad_properties: AadPropertiesOperations operations
    :vartype aad_properties:
     azure.mgmt.recoveryservicesbackup.passivestamp.aio.operations.AadPropertiesOperations
    :ivar cross_region_restore: CrossRegionRestoreOperations operations
    :vartype cross_region_restore:
     azure.mgmt.recoveryservicesbackup.passivestamp.aio.operations.CrossRegionRestoreOperations
    :ivar backup_crr_job_details: BackupCrrJobDetailsOperations operations
    :vartype backup_crr_job_details:
     azure.mgmt.recoveryservicesbackup.passivestamp.aio.operations.BackupCrrJobDetailsOperations
    :ivar backup_crr_jobs: BackupCrrJobsOperations operations
    :vartype backup_crr_jobs:
     azure.mgmt.recoveryservicesbackup.passivestamp.aio.operations.BackupCrrJobsOperations
    :ivar crr_operation_results: CrrOperationResultsOperations operations
    :vartype crr_operation_results:
     azure.mgmt.recoveryservicesbackup.passivestamp.aio.operations.CrrOperationResultsOperations
    :ivar crr_operation_status: CrrOperationStatusOperations operations
    :vartype crr_operation_status:
     azure.mgmt.recoveryservicesbackup.passivestamp.aio.operations.CrrOperationStatusOperations
    :ivar recovery_points: RecoveryPointsOperations operations
    :vartype recovery_points:
     azure.mgmt.recoveryservicesbackup.passivestamp.aio.operations.RecoveryPointsOperations
    :ivar backup_resource_storage_configs: BackupResourceStorageConfigsOperations operations
    :vartype backup_resource_storage_configs:
     azure.mgmt.recoveryservicesbackup.passivestamp.aio.operations.BackupResourceStorageConfigsOperations
    :ivar recovery_points_crr: RecoveryPointsCrrOperations operations
    :vartype recovery_points_crr:
     azure.mgmt.recoveryservicesbackup.passivestamp.aio.operations.RecoveryPointsCrrOperations
    :ivar backup_protected_items_crr: BackupProtectedItemsCrrOperations operations
    :vartype backup_protected_items_crr:
     azure.mgmt.recoveryservicesbackup.passivestamp.aio.operations.BackupProtectedItemsCrrOperations
    :param credential: Credential needed for the client to connect to Azure. Required.
    :type credential: ~azure.core.credentials_async.AsyncTokenCredential
    :param subscription_id: The subscription Id. Required.
    :type subscription_id: str
    :param base_url: Service URL. Default value is None.
    :type base_url: str
    :keyword cloud_setting: The cloud setting for which to get the ARM endpoint. Default value is
     None.
    :paramtype cloud_setting: ~azure.core.AzureClouds
    :keyword api_version: Api Version. Default value is "2021-11-15". Note that overriding this
     default value may result in unsupported behavior.
    :paramtype api_version: str
    :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
     Retry-After header is present.
    """

    def __init__(
        self,
        credential: "AsyncTokenCredential",
        subscription_id: str,
        base_url: Optional[str] = None,
        *,
        cloud_setting: Optional["AzureClouds"] = None,
        **kwargs: Any
    ) -> None:
        _cloud = cloud_setting or settings.current.azure_cloud  # type: ignore
        _endpoints = get_arm_endpoints(_cloud)
        if not base_url:
            base_url = _endpoints["resource_manager"]
        credential_scopes = kwargs.pop("credential_scopes", _endpoints["credential_scopes"])
        self._config = RecoveryServicesBackupPassiveClientConfiguration(
            credential=credential,
            subscription_id=subscription_id,
            cloud_setting=cloud_setting,
            credential_scopes=credential_scopes,
            **kwargs
        )

        _policies = kwargs.pop("policies", None)
        if _policies is None:
            _policies = [
                policies.RequestIdPolicy(**kwargs),
                self._config.headers_policy,
                self._config.user_agent_policy,
                self._config.proxy_policy,
                policies.ContentDecodePolicy(**kwargs),
                AsyncARMAutoResourceProviderRegistrationPolicy(),
                self._config.redirect_policy,
                self._config.retry_policy,
                self._config.authentication_policy,
                self._config.custom_hook_policy,
                self._config.logging_policy,
                policies.DistributedTracingPolicy(**kwargs),
                policies.SensitiveHeaderCleanupPolicy(**kwargs) if self._config.redirect_policy else None,
                self._config.http_logging_policy,
            ]
        self._client: AsyncARMPipelineClient = AsyncARMPipelineClient(
            base_url=cast(str, base_url), policies=_policies, **kwargs
        )

        client_models = {k: v for k, v in _models.__dict__.items() if isinstance(v, type)}
        self._serialize = Serializer(client_models)
        self._deserialize = Deserializer(client_models)
        self._serialize.client_side_validation = False
        self.backup_usage_summaries_crr = BackupUsageSummariesCRROperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.aad_properties = AadPropertiesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.cross_region_restore = CrossRegionRestoreOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.backup_crr_job_details = BackupCrrJobDetailsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.backup_crr_jobs = BackupCrrJobsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.crr_operation_results = CrrOperationResultsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.crr_operation_status = CrrOperationStatusOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.recovery_points = RecoveryPointsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.backup_resource_storage_configs = BackupResourceStorageConfigsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.recovery_points_crr = RecoveryPointsCrrOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.backup_protected_items_crr = BackupProtectedItemsCrrOperations(
            self._client, self._config, self._serialize, self._deserialize
        )

    def _send_request(
        self, request: HttpRequest, *, stream: bool = False, **kwargs: Any
    ) -> Awaitable[AsyncHttpResponse]:
        """Runs the network request through the client's chained policies.

        >>> from azure.core.rest import HttpRequest
        >>> request = HttpRequest("GET", "https://www.example.org/")
        <HttpRequest [GET], url: 'https://www.example.org/'>
        >>> response = await client._send_request(request)
        <AsyncHttpResponse: 200 OK>

        For more information on this code flow, see https://aka.ms/azsdk/dpcodegen/python/send_request

        :param request: The network request you want to make. Required.
        :type request: ~azure.core.rest.HttpRequest
        :keyword bool stream: Whether the response payload will be streamed. Defaults to False.
        :return: The response of your network call. Does not do error handling on your response.
        :rtype: ~azure.core.rest.AsyncHttpResponse
        """

        request_copy = deepcopy(request)
        request_copy.url = self._client.format_url(request_copy.url)
        return self._client.send_request(request_copy, stream=stream, **kwargs)  # type: ignore

    async def close(self) -> None:
        await self._client.close()

    async def __aenter__(self) -> Self:
        await self._client.__aenter__()
        return self

    async def __aexit__(self, *exc_details: Any) -> None:
        await self._client.__aexit__(*exc_details)



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.recoveryservicesbackup.passivestamp.aio import RecoveryServicesBackupPassiveClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer
from devtools_testutils.aio import recorded_by_proxy_async

AZURE_LOCATION = "eastus"


@pytest.mark.live_test_only
class TestRecoveryServicesOperationsAsync(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(RecoveryServicesBackupPassiveClient, is_async=True)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_aad_properties_get(self):
        response = self.client.aad_properties.get(
            azure_region="southeastasia",
        )
        assert response



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from enum import Enum
from azure.core import CaseInsensitiveEnumMeta


class BackupManagementType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Backup management type to execute the current job."""

    INVALID = "Invalid"
    AZURE_IAAS_VM = "AzureIaasVM"
    MAB = "MAB"
    DPM = "DPM"
    AZURE_BACKUP_SERVER = "AzureBackupServer"
    AZURE_SQL = "AzureSql"
    AZURE_STORAGE = "AzureStorage"
    AZURE_WORKLOAD = "AzureWorkload"
    DEFAULT_BACKUP = "DefaultBackup"


class CopyOptions(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Options to resolve copy conflicts."""

    INVALID = "Invalid"
    CREATE_COPY = "CreateCopy"
    SKIP = "Skip"
    OVERWRITE = "Overwrite"
    FAIL_ON_CONFLICT = "FailOnConflict"


class CreateMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Create mode to indicate recovery of existing soft deleted data source or creation of new data
    source.
    """

    INVALID = "Invalid"
    DEFAULT = "Default"
    RECOVER = "Recover"


class DataSourceType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of workload this item represents."""

    INVALID = "Invalid"
    VM = "VM"
    FILE_FOLDER = "FileFolder"
    AZURE_SQL_DB = "AzureSqlDb"
    SQLDB = "SQLDB"
    EXCHANGE = "Exchange"
    SHAREPOINT = "Sharepoint"
    V_MWARE_VM = "VMwareVM"
    SYSTEM_STATE = "SystemState"
    CLIENT = "Client"
    GENERIC_DATA_SOURCE = "GenericDataSource"
    SQL_DATA_BASE = "SQLDataBase"
    AZURE_FILE_SHARE = "AzureFileShare"
    SAP_HANA_DATABASE = "SAPHanaDatabase"
    SAP_ASE_DATABASE = "SAPAseDatabase"


class HealthState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Health State for the backed up item."""

    PASSED = "Passed"
    ACTION_REQUIRED = "ActionRequired"
    ACTION_SUGGESTED = "ActionSuggested"
    INVALID = "Invalid"


class HealthStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """backups running status for this backup item."""

    PASSED = "Passed"
    ACTION_REQUIRED = "ActionRequired"
    ACTION_SUGGESTED = "ActionSuggested"
    INVALID = "Invalid"


class JobOperationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of operation."""

    INVALID = "Invalid"
    REGISTER = "Register"
    UN_REGISTER = "UnRegister"
    CONFIGURE_BACKUP = "ConfigureBackup"
    BACKUP = "Backup"
    RESTORE = "Restore"
    DISABLE_BACKUP = "DisableBackup"
    DELETE_BACKUP_DATA = "DeleteBackupData"
    CROSS_REGION_RESTORE = "CrossRegionRestore"
    UNDELETE = "Undelete"
    UPDATE_CUSTOMER_MANAGED_KEY = "UpdateCustomerManagedKey"


class JobStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Status of the job."""

    INVALID = "Invalid"
    IN_PROGRESS = "InProgress"
    COMPLETED = "Completed"
    FAILED = "Failed"
    COMPLETED_WITH_WARNINGS = "CompletedWithWarnings"
    CANCELLED = "Cancelled"
    CANCELLING = "Cancelling"


class JobSupportedAction(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """JobSupportedAction."""

    INVALID = "Invalid"
    CANCELLABLE = "Cancellable"
    RETRIABLE = "Retriable"


class LastBackupStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Last backup operation status. Possible values: Healthy, Unhealthy."""

    INVALID = "Invalid"
    HEALTHY = "Healthy"
    UNHEALTHY = "Unhealthy"
    IR_PENDING = "IRPending"


class MabServerType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Server type of MAB container."""

    INVALID = "Invalid"
    UNKNOWN = "Unknown"
    IAAS_VM_CONTAINER = "IaasVMContainer"
    IAAS_VM_SERVICE_CONTAINER = "IaasVMServiceContainer"
    DPM_CONTAINER = "DPMContainer"
    AZURE_BACKUP_SERVER_CONTAINER = "AzureBackupServerContainer"
    MAB_CONTAINER = "MABContainer"
    CLUSTER = "Cluster"
    AZURE_SQL_CONTAINER = "AzureSqlContainer"
    WINDOWS = "Windows"
    V_CENTER = "VCenter"
    VM_APP_CONTAINER = "VMAppContainer"
    SQLAG_WORK_LOAD_CONTAINER = "SQLAGWorkLoadContainer"
    STORAGE_CONTAINER = "StorageContainer"
    GENERIC_CONTAINER = "GenericContainer"


class OperationStatusValues(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Operation status."""

    INVALID = "Invalid"
    IN_PROGRESS = "InProgress"
    SUCCEEDED = "Succeeded"
    FAILED = "Failed"
    CANCELED = "Canceled"


class OverwriteOptions(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Can Overwrite if Target DataBase already exists."""

    INVALID = "Invalid"
    FAIL_ON_CONFLICT = "FailOnConflict"
    OVERWRITE = "Overwrite"


class ProtectedItemHealthStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Health status of the backup item, evaluated based on last heartbeat received."""

    INVALID = "Invalid"
    HEALTHY = "Healthy"
    UNHEALTHY = "Unhealthy"
    NOT_REACHABLE = "NotReachable"
    IR_PENDING = "IRPending"


class ProtectedItemState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Backup state of the backed up item."""

    INVALID = "Invalid"
    IR_PENDING = "IRPending"
    PROTECTED = "Protected"
    PROTECTION_ERROR = "ProtectionError"
    PROTECTION_STOPPED = "ProtectionStopped"
    PROTECTION_PAUSED = "ProtectionPaused"


class ProtectionState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Backup state of this backup item."""

    INVALID = "Invalid"
    IR_PENDING = "IRPending"
    PROTECTED = "Protected"
    PROTECTION_ERROR = "ProtectionError"
    PROTECTION_STOPPED = "ProtectionStopped"
    PROTECTION_PAUSED = "ProtectionPaused"


class RecoveryMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Defines whether the current recovery mode is file restore or database restore."""

    INVALID = "Invalid"
    FILE_RECOVERY = "FileRecovery"
    WORKLOAD_RECOVERY = "WorkloadRecovery"


class RecoveryPointTierStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Recovery point tier status."""

    INVALID = "Invalid"
    VALID = "Valid"
    DISABLED = "Disabled"
    DELETED = "Deleted"
    REHYDRATED = "Rehydrated"


class RecoveryPointTierType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Recovery point tier type."""

    INVALID = "Invalid"
    INSTANT_RP = "InstantRP"
    HARDENED_RP = "HardenedRP"
    ARCHIVED_RP = "ArchivedRP"


class RecoveryType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of this recovery."""

    INVALID = "Invalid"
    ORIGINAL_LOCATION = "OriginalLocation"
    ALTERNATE_LOCATION = "AlternateLocation"
    RESTORE_DISKS = "RestoreDisks"
    OFFLINE = "Offline"


class ResourceHealthStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Resource Health Status."""

    HEALTHY = "Healthy"
    TRANSIENT_DEGRADED = "TransientDegraded"
    PERSISTENT_DEGRADED = "PersistentDegraded"
    TRANSIENT_UNHEALTHY = "TransientUnhealthy"
    PERSISTENT_UNHEALTHY = "PersistentUnhealthy"
    INVALID = "Invalid"


class RestorePointQueryType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """RestorePoint type."""

    INVALID = "Invalid"
    FULL = "Full"
    LOG = "Log"
    DIFFERENTIAL = "Differential"
    FULL_AND_DIFFERENTIAL = "FullAndDifferential"
    ALL = "All"
    INCREMENTAL = "Incremental"


class RestorePointType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of restore point."""

    INVALID = "Invalid"
    FULL = "Full"
    LOG = "Log"
    DIFFERENTIAL = "Differential"
    INCREMENTAL = "Incremental"


class RestoreRequestType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Restore Type (FullShareRestore or ItemLevelRestore)."""

    INVALID = "Invalid"
    FULL_SHARE_RESTORE = "FullShareRestore"
    ITEM_LEVEL_RESTORE = "ItemLevelRestore"


class SQLDataDirectoryType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of data directory mapping."""

    INVALID = "Invalid"
    DATA = "Data"
    LOG = "Log"


class StorageType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Storage type."""

    INVALID = "Invalid"
    GEO_REDUNDANT = "GeoRedundant"
    LOCALLY_REDUNDANT = "LocallyRedundant"
    ZONE_REDUNDANT = "ZoneRedundant"
    READ_ACCESS_GEO_ZONE_REDUNDANT = "ReadAccessGeoZoneRedundant"


class StorageTypeState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Locked or Unlocked. Once a machine is registered against a resource, the storageTypeState is
    always Locked.
    """

    INVALID = "Invalid"
    LOCKED = "Locked"
    UNLOCKED = "Unlocked"


class Type(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Backup management type for this container."""

    INVALID = "Invalid"
    BACKUP_PROTECTED_ITEM_COUNT_SUMMARY = "BackupProtectedItemCountSummary"
    BACKUP_PROTECTION_CONTAINER_COUNT_SUMMARY = "BackupProtectionContainerCountSummary"


class UsagesUnit(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Unit of the usage."""

    COUNT = "Count"
    BYTES = "Bytes"
    SECONDS = "Seconds"
    PERCENT = "Percent"
    COUNT_PER_SECOND = "CountPerSecond"
    BYTES_PER_SECOND = "BytesPerSecond"


class WorkloadType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Workload type of backup item."""

    INVALID = "Invalid"
    VM = "VM"
    FILE_FOLDER = "FileFolder"
    AZURE_SQL_DB = "AzureSqlDb"
    SQLDB = "SQLDB"
    EXCHANGE = "Exchange"
    SHAREPOINT = "Sharepoint"
    V_MWARE_VM = "VMwareVM"
    SYSTEM_STATE = "SystemState"
    CLIENT = "Client"
    GENERIC_DATA_SOURCE = "GenericDataSource"
    SQL_DATA_BASE = "SQLDataBase"
    AZURE_FILE_SHARE = "AzureFileShare"
    SAP_HANA_DATABASE = "SAPHanaDatabase"
    SAP_ASE_DATABASE = "SAPAseDatabase"



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from collections.abc import MutableMapping
from io import IOBase
from typing import Any, Callable, IO, Optional, TypeVar, Union, overload
import urllib.parse

from azure.core import AsyncPipelineClient
from azure.core.async_paging import AsyncItemPaged, AsyncList
from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    map_error,
)
from azure.core.pipeline import PipelineResponse
from azure.core.rest import AsyncHttpResponse, HttpRequest
from azure.core.tracing.decorator import distributed_trace
from azure.core.utils import case_insensitive_dict
from azure.mgmt.core.exceptions import ARMErrorFormat

from ... import models as _models
from ..._utils.serialization import Deserializer, Serializer
from ...operations._backup_crr_jobs_operations import build_list_request
from .._configuration import RecoveryServicesBackupPassiveClientConfiguration

T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, dict[str, Any]], Any]]
List = list


class BackupCrrJobsOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.mgmt.recoveryservicesbackup.passivestamp.aio.RecoveryServicesBackupPassiveClient`'s
        :attr:`backup_crr_jobs` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client: AsyncPipelineClient = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config: RecoveryServicesBackupPassiveClientConfiguration = (
            input_args.pop(0) if input_args else kwargs.pop("config")
        )
        self._serialize: Serializer = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize: Deserializer = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @overload
    def list(
        self,
        azure_region: str,
        parameters: _models.CrrJobRequest,
        filter: Optional[str] = None,
        skip_token: Optional[str] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> AsyncItemPaged["_models.JobResource"]:
        """Gets the list of CRR jobs from the target region.

        Gets the list of CRR jobs from the target region.

        :param azure_region: Azure region to hit Api. Required.
        :type azure_region: str
        :param parameters: Backup CRR Job request. Required.
        :type parameters: ~azure.mgmt.recoveryservicesbackup.passivestamp.models.CrrJobRequest
        :param filter: OData filter options. Default value is None.
        :type filter: str
        :param skip_token: skipToken Filter. Default value is None.
        :type skip_token: str
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: An iterator like instance of either JobResource or the result of cls(response)
        :rtype:
         ~azure.core.async_paging.AsyncItemPaged[~azure.mgmt.recoveryservicesbackup.passivestamp.models.JobResource]
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def list(
        self,
        azure_region: str,
        parameters: IO[bytes],
        filter: Optional[str] = None,
        skip_token: Optional[str] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> AsyncItemPaged["_models.JobResource"]:
        """Gets the list of CRR jobs from the target region.

        Gets the list of CRR jobs from the target region.

        :param azure_region: Azure region to hit Api. Required.
        :type azure_region: str
        :param parameters: Backup CRR Job request. Required.
        :type parameters: IO[bytes]
        :param filter: OData filter options. Default value is None.
        :type filter: str
        :param skip_token: skipToken Filter. Default value is None.
        :type skip_token: str
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: An iterator like instance of either JobResource or the result of cls(response)
        :rtype:
         ~azure.core.async_paging.AsyncItemPaged[~azure.mgmt.recoveryservicesbackup.passivestamp.models.JobResource]
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def list(
        self,
        azure_region: str,
        parameters: Union[_models.CrrJobRequest, IO[bytes]],
        filter: Optional[str] = None,
        skip_token: Optional[str] = None,
        **kwargs: Any
    ) -> AsyncItemPaged["_models.JobResource"]:
        """Gets the list of CRR jobs from the target region.

        Gets the list of CRR jobs from the target region.

        :param azure_region: Azure region to hit Api. Required.
        :type azure_region: str
        :param parameters: Backup CRR Job request. Is either a CrrJobRequest type or a IO[bytes] type.
         Required.
        :type parameters: ~azure.mgmt.recoveryservicesbackup.passivestamp.models.CrrJobRequest or
         IO[bytes]
        :param filter: OData filter options. Default value is None.
        :type filter: str
        :param skip_token: skipToken Filter. Default value is None.
        :type skip_token: str
        :return: An iterator like instance of either JobResource or the result of cls(response)
        :rtype:
         ~azure.core.async_paging.AsyncItemPaged[~azure.mgmt.recoveryservicesbackup.passivestamp.models.JobResource]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.JobResourceList] = kwargs.pop("cls", None)

        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})
        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(parameters, (IOBase, bytes)):
            _content = parameters
        else:
            _json = self._serialize.body(parameters, "CrrJobRequest")

        def prepare_request(next_link=None):
            if not next_link:

                _request = build_list_request(
                    azure_region=azure_region,
                    subscription_id=self._config.subscription_id,
                    filter=filter,
                    skip_token=skip_token,
                    api_version=api_version,
                    content_type=content_type,
                    json=_json,
                    content=_content,
                    headers=_headers,
                    params=_params,
                )
                _request.url = self._client.format_url(_request.url)

            else:
                # make call to next link with the client's api-version
                _parsed_next_link = urllib.parse.urlparse(next_link)
                _next_request_params = case_insensitive_dict(
                    {
                        key: [urllib.parse.quote(v) for v in value]
                        for key, value in urllib.parse.parse_qs(_parsed_next_link.query).items()
                    }
                )
                _next_request_params["api-version"] = self._config.api_version
                _request = HttpRequest(
                    "GET", urllib.parse.urljoin(next_link, _parsed_next_link.path), params=_next_request_params
                )
                _request.url = self._client.format_url(_request.url)
                _request.method = "GET"
            return _request

        async def extract_data(pipeline_response):
            deserialized = self._deserialize("JobResourceList", pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return deserialized.next_link or None, AsyncList(list_of_elem)

        async def get_next(next_link=None):
            _request = prepare_request(next_link)

            _stream = False
            pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
                _request, stream=_stream, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                error = self._deserialize.failsafe_deserialize(_models.NewErrorResponse, pipeline_response)
                raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

            return pipeline_response

        return AsyncItemPaged(get_next, extract_data)
